# -*- coding: utf-8 -*-

import urllib2
import json
import base64
import socket
import logging
import traceback
import ssl

# import library
from rospeex_core.validators import accepts, check_wave_data, check_language
from rospeex_core.exceptions import InvalidRequestException
from rospeex_core.exceptions import InvalidResponseException
from rospeex_core.exceptions import SpeechRecognitionException
from rospeex_core.exceptions import RequestTimeoutException
from rospeex_core.sr.client_base import SpeechRecognitionClient

# create logger
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SpeechRecognitionClient_NICT(SpeechRecognitionClient):
    """ SpeechRecognitionClient_NICT class """

    AUDIO_LENGTH = 16000
    FRAMERATE = 16000
    CHANNELS = 1
    SAMPWIDTH = 2
    URL = 'http://rospeex.ucri.jgn-x.jp/nauth_json/jsServices/VoiceTraSR'
    LANGUAGES = ['ja', 'en', 'zh', 'ko']


    @accepts(data=str, language=str, timeout=int)
    def request(self, data, language='ja', timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
        """
        send speech recognition request to server
        @param data: speech binary data
        @type  data: str
        @param language: speech data language
        @type  language: str
        @param timeout: time out time[ms]
        @type  timeout: int
        """
        check_wave_data(data, self.FRAMERATE, self.CHANNELS, self.SAMPWIDTH, self.AUDIO_LENGTH)
        check_language(language, self.LANGUAGES)

        # load voice data and encord to base64
        voice_send = base64.b64encode(data)

        data = {
            'method':'recognize',
            'params':[
                '1.1',
                {'audio': voice_send,
                 'audioType':'audio/x-wav',
                 'voiceType': '*',
                 'language':language,
                 'applicationType':'rospeex'
                }
            ]
        }

        # サーバにリクエストを送信する
        res_data = None
        try:
            req = urllib2.Request(self.URL)
            req.add_header('Content-Type', 'application/json')
            res = urllib2.urlopen(req, json.dumps(data), timeout=timeout)
            res_data = json.loads(res.read())

        except urllib2.URLError as err:
            if isinstance(err.reason, socket.timeout):
                raise RequestTimeoutException('request time out. Exception: %s' % str(err))
            raise InvalidRequestException('request url error. Exception: %s' % str(err))

        except urllib2.HTTPError as err:
            raise InvalidResponseException('http error. %s Exception:%s' % (err.code, err.msg))

        except (ssl.SSLError, socket.timeout) as err:
            raise RequestTimeoutException(str(err))

        except:
            raise SpeechRecognitionException('unknown exception. Traceback: %s' % traceback.format_exc())

        # 結果の取得
        if res_data['error'] is not None:
            if res_data['error'].has_key('faultCode'):
                if res_data['error']['faultCode'] == 'Server.userException':
                    raise InvalidResponseException('the format is not supported.')
            raise InvalidResponseException('server response error. msg:%s'%res_data['error'])

        return res_data['result']
