#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

# import library
from rospeex_core.exceptions import SpeechRecognitionException
from rospeex_core.sr.client_nict import SpeechRecognitionClient_NICT
from rospeex_core.sr.client_google import SpeechRecognitionClient_Google


# create logger
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SREngine(object):
    """ Engine class """
    NICT = 'nict'
    GOOGLE = 'google'


class SpeechRecognitionFactory(object):
    """
    SpeechRecognitionFactory class
    """
    ENGINE_FACTORY = {
        'google': SpeechRecognitionClient_Google,
        'nict': SpeechRecognitionClient_NICT,
    }

    @classmethod
    def create(cls, engine):
        """
        create speech recognition client
        @param engine: speech recognition engine name (NICT|GOOGLE)
        @type  engine: str
        @raise SpeechRecognitionException:
        """
        if engine in cls.ENGINE_FACTORY.keys():
            return cls.ENGINE_FACTORY[engine]()
        else:
            msg = 'target client [{engine}] is not found. Except: {engine_list}'.format(
                engine=engine,
                engine_list=cls.ENGINE_FACTORY.keys()
            )
            raise SpeechRecognitionException(msg)
