# -*- coding: utf-8 -*-

import urllib
import urllib2
import socket
import subprocess
import traceback
import logging


# import library
from rospeex_core.validators import accepts
from rospeex_core.validators import check_language
from rospeex_core.exceptions import InvalidRequestException
from rospeex_core.exceptions import InvalidResponseException
from rospeex_core.exceptions import SpeechSynthesisException
from rospeex_core.exceptions import RequestTimeoutException
from rospeex_core.ss.client_base import SpeechSynthesisClient
from rospeex_core.ss.client_nict import SpeechSynthesisClient_NICT


logger = logging.getLogger(__name__)


class SpeechSynthesisClient_Google(SpeechSynthesisClient):
    """ SpeechSynthesisCient_Google class """
    LANGUAGES = ['ja', 'en', '*']

    def __init__(self):
        """ init function """
        self.URL = 'http://translate.google.com/translate_tts?'

    @accepts(message=basestring, language=str, voice_font=str, timeout=int)
    def request(self, message, language='ja', voice_font='', timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
        """
        Send speech synthesis request to server,
        and get speech synthesis result.
        @param message: message
        @type  message: str
        @param language: speech synthesis language
        @type  language: str
        @param voice_font: taraget voice font
        @type  voice_font: str
        @param timeout: request timeout time (second)
        @type  timeout: float
        @return: voice data (wav format binary)
        @rtype: str
        @raise SpeechSynthesisException:
        """
        check_language(language, self.LANGUAGES)

        query = {
            'q': message.encode('utf-8'),
            'tl': language,
        }
        values = urllib.urlencode(query)
        headers = {'User-Agent':'Mozilla/5.0'}

        try:
            client = SpeechSynthesisClient_NICT()
            client.request(message, language, voice_font, 10)
        except:
            pass

        voice = None
        try:
            request = urllib2.Request(self.URL+values, None, headers)
            response = urllib2.urlopen(request, timeout=timeout)
            voice = response.read()

        except urllib2.URLError as err:
            if isinstance(err.reason, socket.timeout):
                msg = 'request time out. Exception: %s' % str(err)
                raise RequestTimeoutException(msg)
            msg = 'request url error. Exception: %s' % str(err)
            raise InvalidRequestException(msg)

        except urllib2.HTTPError as err:
            msg = 'http error. %s Exception:%s' % (err.code, err.msg)
            raise InvalidResponseException(msg)

        except socket.timeout as err:
            msg = 'request time out. Exception: %s' % str(err)
            raise RequestTimeoutException(msg)

        except:
            msg = 'unknown exception. Traceback: %s' % traceback.format_exc()
            raise SpeechSynthesisException(msg)

        conv_voice = self._convert_mp3_to_wav(voice)
        return conv_voice


    def _convert_mp3_to_wav(self, data):
        """
        convert mp3 to waf file
        @param data: mp3 binary data
        @type  data: str
        @return: wav binary data
        @rtype: str
        @raises SpeechSynthesisException:
        """
        # convert wav to flac data
        try:
            input_file_name = 'ss_tmp.mp3'
            output_file_name = 'ss_tmp.wav'

            fo = open(input_file_name, 'wb')
            fo.write(data)
            fo.close()

            subprocess.check_output(
                ['ffmpeg', '-y', '-i', input_file_name, output_file_name],
                stderr=subprocess.STDOUT
            )

            fi = open(output_file_name, 'rb')
            data = fi.read()
            fi.close()

        except IOError as err:
            msg = 'file io error. Exception:%s' % str(err)
            raise SpeechSynthesisException(msg)

        except OSError as err:
            msg = 'ffmpeg is not installed. Exception:%s' % str(err)
            raise SpeechSynthesisException(msg)

        except subprocess.CalledProcessError as err:
            msg = 'ffmpeg return error value. Exception:%s' % str(err)
            raise SpeechSynthesisException(msg)

        except:
            msg = 'unknown exception. Traceback: %s' % traceback.format_exc()
            raise SpeechSynthesisException(msg)

        return data
