# -*- coding: utf-8 -*-

import logging

# import library
from rospeex_core.exceptions import SpeechSynthesisException
from rospeex_core.ss.client_nict import SpeechSynthesisClient_NICT
from rospeex_core.ss.client_google import SpeechSynthesisClient_Google
# from rospeex_core.ss.client_voicetext import SpeechSynthesisClient_VoiceText


logger = logging.getLogger(__name__)


class SSEngine(object):
    """ engine enum class """
    NICT = 'nict'
    GOOGLE = 'google'
    # VOICETEXT = 'voicetext'


class SpeechSynthesisFactory(object):
    """ SpeechSynthesisFactory class """
    ENGINE_FACTORY = {
        'google': SpeechSynthesisClient_Google,
        'nict': SpeechSynthesisClient_NICT,
        # 'voicetext': SpeechSynthesisClient_VoiceText,
    }

    @classmethod
    def create(cls, engine=''):
        """
        create speech synthesis engine.
        @param engine: speech synthesis engine (nict / google / voicetext)
        @type  engine: str
        @raise SpeechSynthesisException:
        """
        if engine in cls.ENGINE_FACTORY.keys():
            return cls.ENGINE_FACTORY[engine]()
        else:
            msg = 'target client [{engine}] is not found. Except: {engine_list}'.format(
                engine=engine,
                engine_list=cls.ENGINE_FACTORY.keys()
            )
            raise SpeechSynthesisException(msg)
